include(AmdisCXXFeatures)
include(AddAmdisExecutable)

# some optimization of the compile times
find_program(CCACHE_PROGRAM ccache)
if (CCACHE_PROGRAM)
  set(CMAKE_CXX_COMPILER_LAUNCHER "${CCACHE_PROGRAM}")
endif ()

if (UNIX AND NOT APPLE)
  execute_process(COMMAND ${CMAKE_C_COMPILER} -fuse-ld=gold -Wl,--version ERROR_QUIET OUTPUT_VARIABLE ld_version)
  if ("${ld_version}" MATCHES "GNU gold")
    set(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -fuse-ld=gold -Wl,--disable-new-dtags")
    set(CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -fuse-ld=gold -Wl,--disable-new-dtags")
  endif()
endif()


if (NOT BACKEND)
  set(BACKEND "ISTL" CACHE STRING "LinearAlgebra backend. One of MTL, EIGEN, PETSC, ISTL")
  set_property(CACHE BACKEND PROPERTY STRINGS "MTL" "EIGEN" "ISTL" "PETSC")
endif (NOT BACKEND)

if (BACKEND STREQUAL "MTL" OR BACKEND STREQUAL "MTL4")
  find_package(MTL REQUIRED)
  set(HAVE_MTL TRUE)
  message(STATUS "  Found MTL, version: ${MTL_VERSION}")
  dune_register_package_flags(LIBRARIES MTL::MTL COMPILE_DEFINITIONS "ENABLE_MTL=1")

  find_package(HYPRE)
  if (HYPRE_FOUND)
    set(HAVE_HYPRE TRUE)
    dune_register_package_flags(LIBRARIES HYPRE::HYPRE COMPILE_DEFINITIONS "ENABLE_HYPRE=1")
  endif(HYPRE_FOUND)
elseif (BACKEND STREQUAL "EIGEN" OR BACKEND STREQUAL "EIGEN3")
  find_package(Eigen3 REQUIRED 3.3.5)
  set(HAVE_EIGEN TRUE)
  if (EIGEN3_FOUND)
    message(STATUS "  Found Eigen3, version: ${Eigen3_VERSION}")
    dune_register_package_flags(LIBRARIES Eigen3::Eigen COMPILE_DEFINITIONS "ENABLE_EIGEN=1")
  endif (EIGEN3_FOUND)
elseif (BACKEND STREQUAL "PETSC")
  find_package(PETSc REQUIRED)
  if (PETSc_FOUND)
    set(HAVE_PETSC TRUE)
    dune_register_package_flags(LIBRARIES PETSc::PETSc COMPILE_DEFINITIONS "ENABLE_PETSC=1")
  endif (PETSc_FOUND)
elseif (BACKEND STREQUAL "ISTL")
  if (NOT dune-istl_FOUND)
    message(FATAL_ERROR "Need dune-istl, MTL, PETSc, or Eigen3 as linear algebra backend. Change flag BACKEND!")
  endif ()
else ()
  message(FATAL_ERROR "BACKEND must be one of MTL, EIGEN, PETSC, ISTL")
endif ()


set(DUNE_CUSTOM_PKG_CONFIG_SECTION "set(BACKEND \"${BACKEND}\")")