#pragma once

#include <memory>

#include <amdis/Boundary.hpp>
#include <amdis/BoundaryManager.hpp>

namespace AMDiS
{
  class BoundaryCondition
  {
  public:
    BoundaryCondition() = default;
    BoundaryCondition(std::shared_ptr<BoundaryManagerBase const> const& boundaryManager, BoundaryType id)
      : boundaryManager_(boundaryManager)
      , id_(id)
    {}

    /// Return true if intersection is on boundary with id
    template <class Intersection>
    bool onBoundary(Intersection const& is) const
    {
      return is.boundary() && (!boundaryManager_  || boundaryManager_->boundaryId(is) == id_);
    }

    template <class RowBasis, class ColBasis>
    void init(RowBasis const& rowBasis, ColBasis const& colBasis) { /* do nothing */ }

    template <class Matrix, class X, class B, class RN, class RTP, class CN, class CTP>
    void fillBoundaryCondition(Matrix& A, X& x, B& b, RN const& rowNode, RTP rowTreePath, CN const& colNode, CTP colTreePath) { /* do nothing */ }

  protected:
    std::shared_ptr<BoundaryManagerBase const> boundaryManager_{nullptr};
    BoundaryType id_{0};
  };

} // end namespace AMDiS
