#pragma once

#include <amdis/common/Concepts.hpp>

namespace AMDiS
{
  // polynomial order of local functions
  template <class LocalFunction,
    class = void_t<decltype(std::declval<LocalFunction>().order())> >
  int order(LocalFunction const& lf)
  {
    return lf.order();
  }


  namespace Concepts
  {
    /** \addtogroup Concepts
     *  @{
     **/

    namespace Definition
    {
      struct HasLocalFunctionOrder
      {
        template <class F>
        auto require(F&& f) -> decltype(
          order(localFunction(f))
        );
      };

      struct HasOrder
      {
        template <class F>
        auto require(F&& f) -> decltype(
          order(f)
        );
      };

    } // end namespace Definition


    /// \brief GridFunction GF has free function `order(localFunction(F))`
    template <class GF>
    constexpr bool HasLocalFunctionOrder = models<Definition::HasLocalFunctionOrder(GF)>;

    template <class GF>
    using HasLocalFunctionOrder_t = models_t<Definition::HasLocalFunctionOrder(GF)>;


    /// \brief LocalFuncion LF has free function `order(F)`
    template <class LF>
    constexpr bool HasOrder = models<Definition::HasOrder(LF)>;

    template <class LF>
    using HasOrder_t = models_t<Definition::HasOrder(LF)>;

    /** @} **/

  } // end namespace Concepts

} // end namespace AMDiS
