#pragma once

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/grid/common/partitionset.hh>
#include <dune/istl/bcrsmatrix.hh>
#include <dune/istl/bvector.hh>
#include <dune/istl/operators.hh>
#include <dune/istl/preconditioner.hh>
#include <dune/istl/scalarproducts.hh>
#include <dune/istl/solver.hh>

#include <amdis/linearalgebra/istl/Communication.hpp>
#include <amdis/linearalgebra/istl/Creators.hpp>

namespace AMDiS
{
  template <class T, class = void>
  struct BlockMatrixType
  {
    using type = Dune::FieldMatrix<T,1,1>;
  };

  template <class T>
  struct BlockMatrixType<T, typename T::field_type>
  {
    using type = T;
  };

  template <class T, class = void>
  struct BlockVectorType
  {
    using type = Dune::FieldVector<T,1>;
  };

  template <class T>
  struct BlockVectorType<T, typename T::field_type>
  {
    using type = T;
  };


  /** Traits class for a linear solver for the system AX=B using an FE space described by a dune-functions Basis
   *  Contains typedefs specific to the ISTL backend.
   */
  template <class Basis, class T = double>
  struct BackendTraits
  {
    using Mat             = Dune::BCRSMatrix<typename BlockMatrixType<T>::type>;
    using Vec             = Dune::BlockVector<typename BlockVectorType<T>::type>;
    using CoefficientType = T;
    using Comm            = ISTLCommunication<Basis>;
    using PartitionSet    = Dune::Partitions::All;

    using ScalProd        = Dune::ScalarProduct<Vec>;
    using LinOp           = Dune::AssembledLinearOperator<Mat, Vec, Vec>;
    using Solver          = Dune::InverseOperator<Vec, Vec>;
    using Prec            = Dune::Preconditioner<Vec, Vec>;
    using ScalProdCreator = ISTLScalarProductCreator<Vec>;
    using ParPrecCreator  = ISTLParallelPreconditionerCreator<Vec>;
    using LinOpCreator    = ISTLLinearOperatorCreator<Mat, Vec>;
  };

  template <class Traits>
  struct SeqBackendTraits : Traits
  {
    using Comm = typename Traits::Comm::Sequential;
  };

} // end namespace AMDiS
