#pragma once

#include <boost/numeric/mtl/matrix/compressed2D.hpp>
#include <boost/numeric/mtl/vector/dense_vector.hpp>

#include <dune/grid/common/partitionset.hh>
#include <amdis/linearalgebra/Communication.hpp>

namespace AMDiS
{
  /** Traits class for a linear solver for the system AX=B using an FE space described by a dune-functions Basis
   *  Contains typedefs specific to the MTL backend.
   */
  template <class Basis, class T = double>
  struct BackendTraits
  {
    using Mat = mtl::compressed2D<T>;
    using Vec = mtl::dense_vector<T>;
    using CoefficientType = T;
    using Comm = SequentialCommunication;
    using PartitionSet = Dune::Partitions::All;
  };

} // end namespace AMDiS
