#pragma once

#include <algorithm>
#include <vector>

#if HAVE_MPI
#include <amdis/common/parallel/Communicator.hpp>
#endif

namespace AMDiS
{
  /// Sparsity pattern used to create PETSc matrices
  class MatrixNnzStructure
  {
  public:
    // Return Number of nonzeros in the diagonal part (owner part)
    std::vector<PetscInt> const& d_nnz() const
    {
      return dnnz_;
    }

    /// Return Number of nonzeros in the off-diagonal part (overlap part)
    std::vector<PetscInt> const& o_nnz() const
    {
      return onnz_;
    }

    /// Calculate the matrix pattern from the local pattern of a basis.
    // NOTE: this assumes a square matrix with row-basis = col-basis.
    template <class Basis>
    void update(Basis const& basis);

  private:
    std::vector<PetscInt> dnnz_; //< number of nonzeros in the diagonal part (owner part)
    std::vector<PetscInt> onnz_; //< number of nonzeros in the off-diagonal part (overlap part)

    // use a grid change index to decide whether to update the matrix pattern or not.
    unsigned long changeIndex_ = 0;
    bool initialized_ = false;

#if HAVE_MPI
    const Mpi::Tag tag_{916821}; //< Communication tag used internally
#endif
  };

} // end namespace AMDiS

#include <amdis/linearalgebra/petsc/MatrixNnzStructure.inc.hpp>
