#include <amdis/AMDiS.hpp>
#include <amdis/common/Access.hpp>

#include "Tests.hpp"

using namespace AMDiS;

struct Vec1
{
  int  operator[](std::size_t) const { return value; }
  int& operator[](std::size_t) { return value; }

  int value = 0;
};

struct Vec2
{
  int  operator()(std::size_t) const { return value; }
  int& operator()(std::size_t) { return value; }

  int value = 0;
};

struct Vec3
{
  int operator[](std::size_t) const { return 0; }
  int operator()(std::size_t) const { return 0; }
};

struct Mat1
{
  Vec1 const& operator[](std::size_t) const { return row; }
  Vec1& operator[](std::size_t) { return row; }

  Vec1 row;
};

struct Mat2
{
  int  operator()(std::size_t,std::size_t) const { return value; }
  int& operator()(std::size_t,std::size_t) { return value; }

  int value = 0;
};

struct Mat3
{
  Vec1 operator[](std::size_t) const { return {}; }
  int  operator()(std::size_t,std::size_t) const { return 0; }
};

struct NotAccessible {};

int main(int argc, char** argv)
{
  Environment env(argc, argv);

  Vec1 vec1;
  Vec2 vec2;
  Vec3 vec3;

  access(vec1, 1) = 2;
  int i1 = access(vec1, 1);
  AMDIS_TEST_EQ(i1,2);

  access(vec2, 1) = 2;
  int i2 = access(vec2, 1);
  AMDIS_TEST_EQ(i2,2);

  int i3 = access(vec3, 1);
  AMDIS_TEST_EQ(i3,0);

  static_assert(Concepts::VectorAccessible_t<Vec1,std::size_t>::value, "");
  static_assert(Concepts::VectorAccessible_t<Vec2,std::size_t>::value, "");
  static_assert(Concepts::VectorAccessible_t<Vec3,std::size_t>::value, "");

  Mat1 mat1;
  Mat2 mat2;
  Mat3 mat3;

  access(mat1, 1,1) = 2;
  int j1 = access(mat1, 1,1);
  AMDIS_TEST_EQ(j1,2);

  access(mat2, 1,1) = 2;
  int j2 = access(mat2, 1,1);
  AMDIS_TEST_EQ(j2,2);

  int j3 = access(mat3, 1,1);
  AMDIS_TEST_EQ(j3,0);

  static_assert(Concepts::MatrixAccessible_t<Mat1,std::size_t,std::size_t>::value, "");
  static_assert(Concepts::MatrixAccessible_t<Mat2,std::size_t,std::size_t>::value, "");
  static_assert(Concepts::MatrixAccessible_t<Mat3,std::size_t,std::size_t>::value, "");

  NotAccessible notAccessible;
  auto k1 = access(notAccessible,1);
  auto k2 = access(notAccessible,1,1);
  static_assert(std::is_same<decltype(k1),NotAccessible>::value, "");
  static_assert(std::is_same<decltype(k2),NotAccessible>::value, "");

  static_assert(not Concepts::VectorAccessible_t<NotAccessible,std::size_t>::value, "");
  static_assert(not Concepts::MatrixAccessible_t<NotAccessible,std::size_t,std::size_t>::value, "");

  return 0;
}
